
import matplotlib as mpl
#mpl.use('Agg')
import astropy.units as u
import astropy.constants as const
import numpy
import IPython
import h5py,sys
import matplotlib.pyplot as plt
from matplotlib.colors import LogNorm
from matplotlib import rc
import pylab


rc('font',**{'family':'serif','serif':['Palatino']})
rc('text', usetex=True)
plt.rcParams["xtick.major.size"] = 5
plt.rcParams["ytick.major.size"] = 5

def SetLabels(xsize=18,ysize=18):
    ax = pylab.gca()
    for tick in ax.xaxis.get_major_ticks():
        tick.label1.set_fontsize(xsize)

    for tick in ax.yaxis.get_major_ticks():
        tick.label1.set_fontsize(ysize)


dt = 0.0001

G = 1.0
M = 1.0

X0,Y0,Z0 = 0.0,0.0,0.0
X1,Y1,Z1 = 5.0,15.0,0.0

VX0,VY0,VZ0 = 0.0,0.0,0.0
VX1,VY1,VZ1 = 0.0,-0.7,0.0


N = 72
Ncircles = 5

Theta = numpy.linspace(0.0,Ncircles*2*numpy.pi, N,endpoint=False)
R = numpy.linspace(2,5.0,N)
#R = numpy.zeros(N)+5.0

x = R * numpy.cos(Theta)
y = R * numpy.sin(Theta)
z = numpy.zeros(N)

vtheta = numpy.sqrt(G*M/R)
vr = 0.0

vx = vr * numpy.cos(Theta) - numpy.sin(Theta) * vtheta
vy = vr * numpy.sin(Theta) + numpy.cos(Theta) * vtheta
vz = numpy.zeros(N)

m = x*0.0+0.00001
m =  numpy.append(m,M)
m =  numpy.append(m,0.5)


x =  numpy.append(x,X0)
x =  numpy.append(x,X1)

y =  numpy.append(y,Y0)
y =  numpy.append(y,Y1)

z =  numpy.append(z,Z0)
z =  numpy.append(z,Z1)

vx =  numpy.append(vx,VX0)
vx =  numpy.append(vx,VX1)

vy =  numpy.append(vy,VY0)
vy =  numpy.append(vy,VY1)

vz =  numpy.append(vz,VZ0)
vz =  numpy.append(vz,VZ1)


Leapfrog = False

if Leapfrog == True:
    #Force update
    ax = -G*M/(x**2+y**2+z**2)**1.5 * x
    ay = -G*M/(x**2+y**2+z**2)**1.5 * y
    az = -G*M/(x**2+y**2+z**2)**1.5 * z

    #velocity update
    vx = vx - ax*dt/2.0
    vy = vy - ay*dt/2.0
    vz = vz - az*dt/2.0


for Ntimestep in range(100000):
    #Convention: dx[i,j] = x[i] - x[j]
    N = x.shape[0]
    dx = numpy.repeat(x,N) - numpy.tile(x,N)
    dy = numpy.repeat(y,N) - numpy.tile(y,N)
    dz = numpy.repeat(z,N) - numpy.tile(z,N)
    dx.shape = (N,N)
    dy.shape = (N,N)
    dz.shape = (N,N)

    #r2_ij, r_ij and m2_ij are calculated:
    r2_ij = dx**2+dy**2+dz**2

    r_ij = numpy.sqrt(r2_ij)
    m2_ij = numpy.outer(m,m)

    #Fij is used to compute dvx, dvy and dvz in an efficient way:
    F_ij = G*m2_ij / r2_ij / r_ij

    ax = numpy.sum(F_ij * dx,axis=0) / m
    ay = numpy.sum(F_ij * dy,axis=0) / m
    az = numpy.sum(F_ij * dz,axis=0) / m

    GoodIDs = numpy.where(m==0.0)

    #Force update
#    ax = -G*M/(x**2+y**2+z**2)**1.5 * x
#    ay = -G*M/(x**2+y**2+z**2)**1.5 * y
#    az = -G*M/(x**2+y**2+z**2)**1.5 * z

    #velocity update
    vx = vx + ax*dt
    vy = vy + ay*dt
    vz = vz + az*dt

    #position update
    x = x + vx*dt
    y = y + vy*dt
    z = z + vz*dt


    if Ntimestep%50000==0:
        plt.plot(x[0:-3],y[0:-3],'-o',mew=0)
        MassiveIDs = numpy.where(m>0.001)
        plt.plot(x[MassiveIDs],y[MassiveIDs],'o',ms=10,mew=0,color='black')
#        if Leapfrog:
#            plt.quiver(x, y, vx+ax*dt/2.0, vy+ay*dt/2.0, units='width',color='blue')
#        else:
#            plt.quiver(x, y, vx+ax, vy, units='width',color='blue')

#        ax = -G*M/(x**2+y**2+z**2)**1.5 * x
#        ay = -G*M/(x**2+y**2+z**2)**1.5 * y
#        az = -G*M/(x**2+y**2+z**2)**1.5 * z

#        plt.quiver(x, y, ax, ay, units='width',color='red')

        plt.xlim((-20,20))
        plt.ylim((-20,20))
        plt.show()

sys.exit()



#plt.figure(1)
#plt.figure(figsize=(15,15))
#plt.subplots_adjust(left=0.15,right=0.96, top=0.92,wspace=0.0, hspace=0.0,bottom=0.15)
#plt.subplot(2,2,1)
#plt.xlabel('$x$',fontsize=16)
#plt.ylabel('$y$',fontsize=16)
#plt.xlim((1.0,3.0))
#plt.ylim((0.0,10.0))
#plt.savefig(sys.argv[0].strip('.py')+'_.png')
#plt.savefig(sys.argv[0].strip('.py')+'_.pdf')
#plt.show()
#plt.clf()
