import matplotlib as mpl
import numpy
import matplotlib.pyplot as plt
#written by Martin Sparre, copyright 2018

#we will work in natural units with G=1
G = 1.0

#Initial conditions
M0 = 1.0
M1 = 1e-6
X0,Y0,Z0 = 0.0,0.0,0.0
X1,Y1,Z1 = 1.0, 0.0,0.0
VX0,VY0,VZ0 = 0.0,0.0,0.0
VX1,VY1,VZ1 = 0.0,numpy.sqrt(G*M0/X1),0.0

#Time integration parameters
t = 0.0#initial time
OrbitalPeriod = 2*numpy.pi*X1/VY1
tmax = 10.0*OrbitalPeriod
dt = 0.05*OrbitalPeriod
Leapfrog = True#if True Leapfrog is enabled, if False it is disabled

#Create offset if Leapfrog method is enabled (if Leapfrog==False we will just use the Euler method)
if Leapfrog == True:
    AX0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X0-X1)
    AY0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y0-Y1)
    AZ0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z0-Z1)

    AX1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X1-X0)
    AY1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y1-Y0)
    AZ1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z1-Z0)

    VX0 = VX0 - AX0*dt/2.0
    VY0 = VY0 - AY0*dt/2.0
    VZ0 = VZ0 - AZ0*dt/2.0

    VX1 = VX1 - AX1*dt/2.0
    VY1 = VY1 - AY1*dt/2.0
    VZ1 = VZ1 - AZ1*dt/2.0

#Main loop of integration
List_X0 = []
List_Y0 = []
List_X1 = []
List_Y1 = []
List_Time = []


while t<= tmax:
    #save coordinates to list so we can plot them later
    List_Time.append(t)
    List_X0.append(X0)
    List_Y0.append(Y0)
    List_X1.append(X1)
    List_Y1.append(Y1)

    #Calculate acceleration
    AX0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X0-X1)
    AY0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y0-Y1)
    AZ0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z0-Z1)

    AX1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X1-X0)
    AY1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y1-Y0)
    AZ1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z1-Z0)

    #Advance velocity
    VX0 = VX0 + AX0*dt
    VY0 = VY0 + AY0*dt
    VZ0 = VZ0 + AZ0*dt

    VX1 = VX1 + AX1*dt
    VY1 = VY1 + AY1*dt
    VZ1 = VZ1 + AZ1*dt

    #Advance positions
    X0 = X0+VX0*dt
    Y0 = Y0+VY0*dt
    Z0 = Z0+VZ0*dt

    X1 = X1+VX1*dt
    Y1 = Y1+VY1*dt
    Z1 = Z1+VZ1*dt

    t=t+dt

#convert lists to numpre arrays:
List_Time = numpy.array(List_Time)
List_X0 = numpy.array(List_X0)
List_Y0 = numpy.array(List_Y0)
List_X1 = numpy.array(List_X1)
List_Y1 = numpy.array(List_Y1)


#Plot orbits
plt.figure(1)
ax = plt.subplot(1,1,1)
cmap_colors = plt.get_cmap('spectral')
ax.plot(List_X0, List_Y0,'-',lw=1,color='black',zorder=5)
ax.plot(List_X1, List_Y1,'-',lw=1,color='black',zorder=5)
ax.scatter(List_X0, List_Y0,c=List_Time/OrbitalPeriod,edgecolors='none',zorder=6)
sc=plt.scatter(List_X1,List_Y1,c=List_Time/OrbitalPeriod,edgecolors='none',zorder=6,s=20)
color_bar = plt.colorbar(sc)
color_bar.set_label( r'$t/$OrbitalPeriod', color='black', fontsize=16)

plt.xlim((-1.5,1.5))
plt.ylim((-1.5,1.5))
plt.xlabel('$x$',fontsize=16)
plt.ylabel('$y$',fontsize=16)
plt.show()
