import matplotlib as mpl
import numpy
import matplotlib.pyplot as plt
import IPython

#we will work in natural units with G=1
G = 1.0

#Initial conditions
M0 = 1.0
M1 = 1.0


X1,Y1,Z1 = 0.0,40.0,0.0
X0,Y0,Z0 = 0.0,0.0,0.0

X0,Y0,Z0 = 0.0,-M1/M0*Y1,0.0

#Set up a Kepler orbit with Y1 as the aphelium distance and ellipticity=0.8:
r_aphelium = Y1
e = 0.8#

#Kepler law machinery:
a = r_aphelium/(1.0+e)
mu = M0*M1/(M0+M1)
v_aphelium = numpy.sqrt( (1.0-e)*mu /( (1.0+e)*a  ) )

#a0 = M1/(M0+M1) * a
#a1 = M1/(M0+M1) * a

#vap0 = numpy.sqrt( (1.0-e)/(1.0+e) * mu/a   )
#vap1 = numpy.sqrt( (1.0-e)/(1.0+e) * mu/a   )

#vap0 = numpy.sqrt( (1.0-e)/(1.0+e) * M1/a0   )
#vap1 = numpy.sqrt( (1.0-e)/(1.0+e) * M0/a1   )


v_aphelium = 0.1*numpy.sqrt(G*M0/numpy.fabs(Y1-Y0))
#IPython.embed()
#Set velocities
VX1,VY1,VZ1 = v_aphelium,0.0,0.0
VX0,VY0,VZ0 = -M1/M0*v_aphelium,0.0,0.0

VX1,VY1,VZ1 = v_aphelium,0.0,0.0
VX0,VY0,VZ0 = -M1/M0*v_aphelium,0.0,0.0


#VX1,VY1,VZ1 = M0/(M1)*v_aphelium,0.0,0.0
#VX0,VY0,VZ0 = -M1/(M0)*v_aphelium,0.0,0.0

#VX1,VY1,VZ1 = 2*M0/(M0+M1)*v_aphelium,0.0,0.0
#VX0,VY0,VZ0 = -2*M1/(M0+M1)*v_aphelium,0.0,0.0

#VX1,VY1,VZ1 = vap0,0.0,0.0
#VX0,VY0,VZ0 = -vap1,0.0,0.0


Vcm = (M0*VX0 + M1*VX1)/(M0+M1)

#VX0 -= Vcm
#VX1 -= Vcm


#OrbitalPeriod = numpy.sqrt(   4*numpy.pi**2/(G*M1)   * a1**3)


a = Y1 - Y0
OrbitalPeriod = numpy.sqrt(   4*numpy.pi**2 * a**3  /(G*(M0+M1)) )
OrbitalPeriod = 0.1*Y1 / v_aphelium
#OrbitalPeriod = numpy.sqrt(   4*numpy.pi**2/(G*(M0+M1))   * a**3)

#we start out from aphelium, so set the velocity accordingly

#Time integration parameters
t = 0.0#initial time
#OrbitalPeriod = ((Y1/VX1)**2)**0.5
tmax = 1.0*OrbitalPeriod
dt = 0.000010*OrbitalPeriod
Leapfrog = True#if True Leapfrog is enabled, if False it is disabled

#Output times
OutputTimes = [0.45*tmax,0.50*tmax,0.51*tmax,0.52*tmax,0.53*tmax,0.54*tmax,0.55*tmax]
#OutputTimes = [0.3*tmax,0.40*tmax,0.5*tmax,0.6*tmax,0.7*tmax,0.8*tmax,0.9*tmax]
#OutputTimes = [0.3*tmax,0.4*tmax,0.42*tmax,0.44*tmax,0.45*tmax,0.46*tmax,0.48*tmax,0.5*tmax,0.55*tmax]

#Disk properties
N = 512
Ncircles = 5
Theta = numpy.linspace(0.0,Ncircles*2*numpy.pi, N,endpoint=False)
R = numpy.linspace(1,5.0,N)


x = R * numpy.cos(Theta) + X0
y = R * numpy.sin(Theta) + Y0
z = numpy.zeros(N) + Z0

vtheta =  - numpy.sqrt(G*M0/R)#co-rotation
#vtheta = numpy.sqrt(G*M0/R)#co-rotation

vr = 0.0

vx = vr * numpy.cos(Theta) - numpy.sin(Theta) * vtheta + VX0
vy = vr * numpy.sin(Theta) + numpy.cos(Theta) * vtheta  + VY0
vz = numpy.zeros(N)  + VZ0


Output_i = 0

plt.figure(1,figsize=(4*len(OutputTimes),4))
plt.subplots_adjust(left=0.03,right=0.99, top=0.92,wspace=0.1, hspace=0.0,bottom=0.15)


#Create offset if Leapfrog method is enabled (if Leapfrog==False we will just use the Euler method)
if Leapfrog == True:
    ax = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (x-X0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (x-X1)
    ay = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (y-Y0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (y-Y1)
    az = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (z-Z0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (z-Z1)


    AX0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X0-X1)
    AY0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y0-Y1)
    AZ0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z0-Z1)

    AX1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X1-X0)
    AY1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y1-Y0)
    AZ1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z1-Z0)

    vx = vx - ax*dt/2.0
    vy = vy - ay*dt/2.0
    vz = vz - az*dt/2.0

    VX0 = VX0 - AX0*dt/2.0
    VY0 = VY0 - AY0*dt/2.0
    VZ0 = VZ0 - AZ0*dt/2.0

    VX1 = VX1 - AX1*dt/2.0
    VY1 = VY1 - AY1*dt/2.0
    VZ1 = VZ1 - AZ1*dt/2.0

#Main loop of integration
List_X0 = []
List_Y0 = []
List_X1 = []
List_Y1 = []
List_Time = []




while t<= tmax:
    #save coordinates to list so we can plot them later
    List_Time.append(t)
    List_X0.append(X0)
    List_Y0.append(Y0)
    List_X1.append(X1)
    List_Y1.append(Y1)

    #Calculate acceleration
    ax = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (x-X0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (x-X1)
    ay = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (y-Y0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (y-Y1)
    az = -G*M0/((x-X0)**2+(y-Y0)**2+(z-Z0)**2)**1.5 * (z-Z0) - G*M1/((x-X1)**2+(y-Y1)**2+(z-Z1)**2)**1.5 * (z-Z1)

    AX0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X0-X1)
    AY0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y0-Y1)
    AZ0 = - G*M1/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z0-Z1)

    AX1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (X1-X0)
    AY1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Y1-Y0)
    AZ1 = - G*M0/((X0-X1)**2+(Y0-Y1)**2+(Z0-Z1)**2)**1.5 * (Z1-Z0)

    #Advance velocity
    vx = vx + ax*dt
    vy = vy + ay*dt
    vz = vz + az*dt

    VX0 = VX0 + AX0*dt
    VY0 = VY0 + AY0*dt
    VZ0 = VZ0 + AZ0*dt

    VX1 = VX1 + AX1*dt
    VY1 = VY1 + AY1*dt
    VZ1 = VZ1 + AZ1*dt

    #Advance positions
    x = x + vx*dt
    y = y + vy*dt
    z = z + vz*dt

    X0 = X0+VX0*dt
    Y0 = Y0+VY0*dt
    Z0 = Z0+VZ0*dt

    X1 = X1+VX1*dt
    Y1 = Y1+VY1*dt
    Z1 = Z1+VZ1*dt

    t=t+dt

    if Output_i < len(OutputTimes):
        if t>OutputTimes[Output_i]:
            plt.subplot(1,len(OutputTimes),Output_i+1)
            plt.title(r"$t=%2.2f t_{\rm max}$"%(OutputTimes[Output_i]/tmax))
            plt.plot(X0,Y0,'o',mew=0,ms=8,color='orange',zorder=5)
            plt.plot(X1,Y1,'o',mew=0,ms=8,color='red',zorder=5)
            plt.plot(x,y,'o',mew=0,ms=2,color='blue',zorder=4)
            #do
            Output_i += 1
            plt.xlim((-75,75))
            plt.ylim((-50,100))

#plt.tight_layout()
#plt.show()
#convert lists to numpre arrays:
List_Time = numpy.array(List_Time)
List_X0 = numpy.array(List_X0)
List_Y0 = numpy.array(List_Y0)
List_X1 = numpy.array(List_X1)
List_Y1 = numpy.array(List_Y1)


#Plot orbits
plt.figure(2)
plt.title('Orbit of the two galaxies')
ax = plt.subplot(1,1,1)
cmap_colors = plt.get_cmap('spectral')
ax.plot(List_X0, List_Y0,'-',lw=1,color='black',zorder=5)
ax.plot(List_X1, List_Y1,'-',lw=1,color='black',zorder=5)
ax.scatter(List_X0, List_Y0,c=List_Time/OrbitalPeriod,edgecolors='none',zorder=6)
sc=plt.scatter(List_X1,List_Y1,c=List_Time/OrbitalPeriod,edgecolors='none',zorder=6,s=20)
color_bar = plt.colorbar(sc)
color_bar.set_label( r'$t/$OrbitalPeriod', color='black', fontsize=16)

plt.xlim((-110.0,110.0))
plt.ylim((-110.0,110.0))
plt.xlabel('$x$',fontsize=16)
plt.ylabel('$y$',fontsize=16)
plt.show()
